==> commerce_shipping_colissimo-1.0.0.tar.gz: commerce_shipping_colissimo/src/Labels.php <==
<?php

namespace Drupal\commerce_shipping_colissimo;

use Drupal\address\AddressInterface;
use Drupal\commerce_shipping\Entity\ShipmentInterface;
use Drupal\commerce_shipping_colissimo\Api\ColissimoApiException;
use Drupal\commerce_shipping_colissimo\Api\LabelApi;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationAddress;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationAddressee;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationLetter;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationOutputFormat;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationParcel;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationPayload;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationSender;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationService;
use Drupal\commerce_shipping_colissimo\Plugin\Commerce\ShippingMethod\Colissimo;
use Drupal\commerce_shipping_label\RemoteLabelFile;
use Drupal\commerce_shipping_label\RemoteShipment;
use Drupal\commerce_shipping_label\ShippingLabelGenerationException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Utility\Error;
use Drupal\file\FileRepositoryInterface;
use Drupal\physical\WeightUnit;
use GuzzleHttp\Exception\GuzzleException;

class Labels {
  use StringTranslationTrait;

  private LabelApi $labelApi;

  private Settings $settings;

  private ConfigFactoryInterface $configFactory;

  private LoggerChannelInterface $logger;

  private FileRepositoryInterface $fileRepository;

  public function __construct(LabelApi $labelApi,
                              Settings $settings,
                              ConfigFactoryInterface $configFactory,
                              LoggerChannelInterface $logger,
                              FileRepositoryInterface $fileRepository) {
    $this->labelApi = $labelApi;
    $this->settings = $settings;
    $this->configFactory = $configFactory;
    $this->logger = $logger;
    $this->fileRepository = $fileRepository;
  }

  /**
   * @throws \Drupal\commerce_shipping_label\ShippingLabelGenerationException
   */
  public function generate(ShipmentInterface $shipment): Label {
    $payload = new LabelGenerationPayload();
    $payload->letter = $this->buildLetter($shipment);
    $payload->outputFormat = $this->buildOutputFormat();

    try {
      $response = $this->labelApi->generate($payload);
    }
    catch (ColissimoApiException $e) {
      throw new ShippingLabelGenerationException('Colissimo: ' . $e->getMessage());
    }
    catch (GuzzleException $e) {
      $this->logger->error(Error::renderExceptionSafe($e));
      throw new ShippingLabelGenerationException($this->t('Failed to generate colissimo label.'));
    }
    if (!$response->labelV2Response
      || !$response->labelV2Response->parcelNumber
      || !$response->files) {
      throw new ShippingLabelGenerationException($this->t('Colissimo failed to return a parcel number or label file.'));
    }
    $parcelNumber = $response->labelV2Response->parcelNumber;

    $file = $this->fileRepository->writeData(
      $response->files[0],
      'temporary://colissimo_label_' . $parcelNumber . '.' . $this->settings->get()->getLabelFormat()
    );

    return new Label(
      new RemoteLabelFile($file->getFileUri(), $this->t('Colissimo Label')),
      new RemoteShipment($parcelNumber, $parcelNumber));
  }

  private function buildLetter(ShipmentInterface $shipment): LabelGenerationLetter {
    $letter = new LabelGenerationLetter();
    $letter->service = $this->buildService($shipment);
    $letter->parcel = $this->buildParcel($shipment);
    $letter->sender = $this->buildSender($shipment);
    $letter->addressee = $this->buildAddressee($shipment);
    return $letter;
  }

  private function buildOutputFormat(): LabelGenerationOutputFormat {
    $format = new LabelGenerationOutputFormat();
    $format->x = 0;
    $format->y = 0;
    $format->outputPrintingType = $this->getOutputPrintintType();
    return $format;
  }

  private function buildService(ShipmentInterface $shipment): LabelGenerationService {
    $service = new LabelGenerationService();
    $method = $shipment->getShippingMethod()->getPlugin();
    assert($method instanceof Colissimo);
    $service->productCode = $this->getProductCode($method);

    $setting = $this->settings->get();
    $depositDate = new \DateTime();
    $depositDate = $depositDate->modify('+' . $setting->getAveragePreparationDelayInDays() . ' days');
    $service->depositDate = $depositDate;

    $service->commercialName = $this->configFactory->get('system.site')->get('name');

    return $service;
  }

  private function buildParcel(ShipmentInterface $shipment): LabelGenerationParcel {
    $parcel = new LabelGenerationParcel();
    $weight = $shipment->getWeight();
    $parcel->weight = $weight ? floatval($weight->convert(WeightUnit::KILOGRAM)->getNumber()) : 0;
    if ($parcel->weight < 0.01) {
      $parcel->weight = $this->settings->get()->getDefaultParcelWeigthInKg();
    }

    if ($this->isRelay($shipment)) {
      $customerProfile = new CustomerProfileWrapper($shipment->getShippingProfile());
      $parcel->pickupLocationId = $customerProfile->getRelayId();
    }

    return $parcel;
  }

  private function isRelay(ShipmentInterface $shipment) : bool {
    $method = $shipment->getShippingMethod()->getPlugin();
    assert($method instanceof Colissimo);
    return $method->isRelay();
  }

  private function buildSender(ShipmentInterface $shipment): LabelGenerationSender {
    $sender = new LabelGenerationSender();
    switch ($this->settings->get()->getLabelSenderParcelIdSource()) {
      case LabelSenderParcelIdSource::ORDER_ID:
        $sender->senderParcelRef = $shipment->getOrder()->id();
        break;

      case LabelSenderParcelIdSource::SHIPMENT_ID:
        $sender->senderParcelRef = $shipment->id();
        break;
    }
    $store = $shipment->getOrder()->getStore();
    $sender->address = $this->buildAddress($store->getAddress());
    $sender->address->companyName = $store->getName();
    $sender->address->email = $store->getEmail();
    return $sender;
  }

  private function buildAddressee(ShipmentInterface $shipment): LabelGenerationAddressee {
    $addressee = new LabelGenerationAddressee();
    $customerProfile = new CustomerProfileWrapper($shipment->getShippingProfile());
    if (!$this->isRelay($shipment)) {
      $addressee->address = $this->buildAddress($customerProfile->getAddressField());
    }
    else {
      $billingProfile = $shipment->getOrder()->getBillingProfile();
      if (!$billingProfile || $billingProfile->get('address')->isEmpty()) {
        throw new ShippingLabelGenerationException($this->t('A billing profile is mandatory to generate a colissimo label with realy pickup.'));
      }
      $address = $billingProfile->get('address')->first();
      assert($address instanceof AddressInterface);
      $addressee->address = $this->buildAddress($address);
    }
    $addressee->address->email = $shipment->getOrder()->getEmail();
    $setting = $this->settings->get();

    if (!$setting->isPhoneFromBillingProfile()) {
      $addressee->address->mobileNumber = $customerProfile->getMobilePhone($setting);
      return $addressee;
    }
    $billingProfile = $shipment->getOrder()->getBillingProfile();
    if (!$billingProfile) {
      return $addressee;
    }
    $addressee->address->mobileNumber = $billingProfile->get($setting->getCustomerProfilePhoneField())->getString();
    return $addressee;
  }

  private function buildAddress(AddressInterface $address): LabelGenerationAddress {
    $labelAddress = new LabelGenerationAddress();
    $labelAddress->companyName = $address->getOrganization();
    $labelAddress->firstName = $address->getGivenName();
    $labelAddress->lastName = $address->getFamilyName();
    $labelAddress->line2 = $address->getAddressLine1();
    $labelAddress->line3 = $address->getAddressLine2();
    $labelAddress->city = $address->getLocality();
    $labelAddress->zipCode = $address->getPostalCode();
    $labelAddress->countryCode = $address->getCountryCode();
    $labelAddress->stateOrProvinceCode = $address->getAdministrativeArea();
    return $labelAddress;
  }

  private function getProductCode(Colissimo $method): string {
    switch ($method->getColissimoType()) {
      case ColissimoType::RELAY:
        return 'A2P';

      case ColissimoType::SIGNATURE:
        return 'DOS';

      case ColissimoType::NO_SIGNATURE:
        return 'DOM';
    }
    throw new \UnexpectedValueException('Unknown colissimo type ' . $method->getColissimoType());
  }

  private function getOutputPrintintType(): string {
    $mapping = [
      LabelFormat::PDF => [
        LabelSize::A4 => 'PDF_10x15_300dpi',
        LabelSize::SIZE_10X15 => 'PDF_10x15_300dpi',
        LabelSize::SIZE_10X10 => 'PDF_10x10_300dpi',
      ],
      LabelFormat::ZPL => [
        LabelSize::SIZE_10X15 => 'ZPL_10x15_300dpi',
        LabelSize::SIZE_10X10 => 'ZPL_10x10_300dpi',
      ],
      LabelFormat::DPL => [
        LabelSize::SIZE_10X15 => 'DPL_10x15_300dpi',
        LabelSize::SIZE_10X10 => 'DPL_10x10_300dpi',
      ],
    ];
    $setting = $this->settings->get();
    /** @var ?string $printingType */
    $printingType = @$mapping[$setting->getLabelFormat()][$setting->getLabelSize()];
    if (!$printingType) {
      throw new \UnexpectedValueException('Incoherent label format/size settings.');
    }
    return $printingType;
  }

}

==> commerce_shipping_colissimo-1.0.1.tar.gz: commerce_shipping_colissimo/src/Labels.php <==
<?php

namespace Drupal\commerce_shipping_colissimo;

use Drupal\address\AddressInterface;
use Drupal\commerce_shipping\Entity\ShipmentInterface;
use Drupal\commerce_shipping_colissimo\Api\ColissimoApiException;
use Drupal\commerce_shipping_colissimo\Api\LabelApi;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationAddress;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationAddressee;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationLetter;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationOutputFormat;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationParcel;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationPayload;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationSender;
use Drupal\commerce_shipping_colissimo\Api\LabelGenerationService;
use Drupal\commerce_shipping_colissimo\Plugin\Commerce\ShippingMethod\Colissimo;
use Drupal\commerce_shipping_label\RemoteLabelFile;
use Drupal\commerce_shipping_label\RemoteShipment;
use Drupal\commerce_shipping_label\ShippingLabelGenerationException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Utility\Error;
use Drupal\file\FileRepositoryInterface;
use Drupal\physical\WeightUnit;
use GuzzleHttp\Exception\GuzzleException;

class Labels {
  use StringTranslationTrait;

  private LabelApi $labelApi;

  private Settings $settings;

  private ConfigFactoryInterface $configFactory;

  private LoggerChannelInterface $logger;

  private FileRepositoryInterface $fileRepository;

  public function __construct(LabelApi $labelApi,
                              Settings $settings,
                              ConfigFactoryInterface $configFactory,
                              LoggerChannelInterface $logger,
                              FileRepositoryInterface $fileRepository) {
    $this->labelApi = $labelApi;
    $this->settings = $settings;
    $this->configFactory = $configFactory;
    $this->logger = $logger;
    $this->fileRepository = $fileRepository;
  }

  /**
   * @throws \Drupal\commerce_shipping_label\ShippingLabelGenerationException
   */
  public function generate(ShipmentInterface $shipment): Label {
    $payload = new LabelGenerationPayload();
    $payload->letter = $this->buildLetter($shipment);
    $payload->outputFormat = $this->buildOutputFormat();

    try {
      $response = $this->labelApi->generate($payload);
    }
    catch (ColissimoApiException $e) {
      throw new ShippingLabelGenerationException('Colissimo: ' . $e->getMessage());
    }
    catch (GuzzleException $e) {
      $this->logger->error(Error::renderExceptionSafe($e));
      throw new ShippingLabelGenerationException($this->t('Failed to generate colissimo label.'));
    }
    if (!$response->labelV2Response
      || !$response->labelV2Response->parcelNumber
      || !$response->files) {
      throw new ShippingLabelGenerationException($this->t('Colissimo failed to return a parcel number or label file.'));
    }
    $parcelNumber = $response->labelV2Response->parcelNumber;

    $file = $this->fileRepository->writeData(
      $response->files[0],
      'temporary://colissimo_label_' . $parcelNumber . '.' . $this->settings->get()->getLabelFormat()
    );

    return new Label(
      new RemoteLabelFile($file->getFileUri(), $this->t('Colissimo Label')),
      new RemoteShipment($parcelNumber, $parcelNumber));
  }

  private function buildLetter(ShipmentInterface $shipment): LabelGenerationLetter {
    $letter = new LabelGenerationLetter();
    $letter->service = $this->buildService($shipment);
    $letter->parcel = $this->buildParcel($shipment);
    $letter->sender = $this->buildSender($shipment);
    $letter->addressee = $this->buildAddressee($shipment);
    return $letter;
  }

  private function buildOutputFormat(): LabelGenerationOutputFormat {
    $format = new LabelGenerationOutputFormat();
    $format->x = 0;
    $format->y = 0;
    $format->outputPrintingType = $this->getOutputPrintintType();
    return $format;
  }

  private function buildService(ShipmentInterface $shipment): LabelGenerationService {
    $service = new LabelGenerationService();
    $method = $shipment->getShippingMethod()->getPlugin();
    assert($method instanceof Colissimo);
    $service->productCode = $this->getProductCode($method);

    $setting = $this->settings->get();
    $depositDate = new \DateTime();
    $depositDate = $depositDate->modify('+' . $setting->getAveragePreparationDelayInDays() . ' days');
    $service->depositDate = $depositDate;

    $service->commercialName = $this->configFactory->get('system.site')->get('name');

    return $service;
  }

  private function buildParcel(ShipmentInterface $shipment): LabelGenerationParcel {
    $parcel = new LabelGenerationParcel();
    $weight = $shipment->getWeight();
    $parcel->weight = $weight ? floatval($weight->convert(WeightUnit::KILOGRAM)->getNumber()) : 0;
    if ($parcel->weight < 0.01) {
      $parcel->weight = $this->settings->get()->getDefaultParcelWeigthInKg();
    }

    if ($this->isRelay($shipment)) {
      $customerProfile = new CustomerProfileWrapper($shipment->getShippingProfile());
      $parcel->pickupLocationId = $customerProfile->getRelayId();
    }

    return $parcel;
  }

  private function isRelay(ShipmentInterface $shipment) : bool {
    $method = $shipment->getShippingMethod()->getPlugin();
    assert($method instanceof Colissimo);
    return $method->isRelay();
  }

  private function buildSender(ShipmentInterface $shipment): LabelGenerationSender {
    $sender = new LabelGenerationSender();
    switch ($this->settings->get()->getLabelSenderParcelIdSource()) {
      case LabelSenderParcelIdSource::ORDER_ID:
        $sender->senderParcelRef = $shipment->getOrder()->id();
        break;

      case LabelSenderParcelIdSource::SHIPMENT_ID:
        $sender->senderParcelRef = $shipment->id();
        break;
    }
    $store = $shipment->getOrder()->getStore();
    $sender->address = $this->buildAddress($store->getAddress());
    $sender->address->companyName = $store->getName();
    $sender->address->email = $store->getEmail();
    return $sender;
  }

  private function buildAddressee(ShipmentInterface $shipment): LabelGenerationAddressee {
    $addressee = new LabelGenerationAddressee();
    $customerProfile = new CustomerProfileWrapper($shipment->getShippingProfile());
    if (!$this->isRelay($shipment)) {
      $addressee->address = $this->buildAddress($customerProfile->getAddressField());
    }
    else {
      $billingProfile = $shipment->getOrder()->getBillingProfile();
      if (!$billingProfile || $billingProfile->get('address')->isEmpty()) {
        throw new ShippingLabelGenerationException($this->t('A billing profile is mandatory to generate a colissimo label with realy pickup.'));
      }
      $address = $billingProfile->get('address')->first();
      assert($address instanceof AddressInterface);
      $addressee->address = $this->buildAddress($address);
    }
    $addressee->address->email = $shipment->getOrder()->getEmail();
    $setting = $this->settings->get();

    if (!$setting->isPhoneFromBillingProfile()) {
      $phoneNumber = new MobilePhoneNumber($customerProfile->getMobilePhone($setting));
      $addressee->address->mobileNumber = $phoneNumber->format();
      return $addressee;
    }
    $billingProfile = $shipment->getOrder()->getBillingProfile();
    if (!$billingProfile) {
      return $addressee;
    }
    $phoneNumber = new MobilePhoneNumber($billingProfile->get($setting->getCustomerProfilePhoneField())->getString());
    $addressee->address->mobileNumber = $phoneNumber->format();
    return $addressee;
  }

  private function buildAddress(AddressInterface $address): LabelGenerationAddress {
    $labelAddress = new LabelGenerationAddress();
    $labelAddress->companyName = $address->getOrganization();
    $labelAddress->firstName = $address->getGivenName();
    $labelAddress->lastName = $address->getFamilyName();
    $labelAddress->line2 = $address->getAddressLine1();
    $labelAddress->line3 = $address->getAddressLine2();
    $labelAddress->city = $address->getLocality();
    $labelAddress->zipCode = $address->getPostalCode();
    $labelAddress->countryCode = $address->getCountryCode();
    $labelAddress->stateOrProvinceCode = $address->getAdministrativeArea();
    return $labelAddress;
  }

  private function getProductCode(Colissimo $method): string {
    switch ($method->getColissimoType()) {
      case ColissimoType::RELAY:
        return 'A2P';

      case ColissimoType::SIGNATURE:
        return 'DOS';

      case ColissimoType::NO_SIGNATURE:
        return 'DOM';
    }
    throw new \UnexpectedValueException('Unknown colissimo type ' . $method->getColissimoType());
  }

  private function getOutputPrintintType(): string {
    $mapping = [
      LabelFormat::PDF => [
        LabelSize::A4 => 'PDF_10x15_300dpi',
        LabelSize::SIZE_10X15 => 'PDF_10x15_300dpi',
        LabelSize::SIZE_10X10 => 'PDF_10x10_300dpi',
      ],
      LabelFormat::ZPL => [
        LabelSize::SIZE_10X15 => 'ZPL_10x15_300dpi',
        LabelSize::SIZE_10X10 => 'ZPL_10x10_300dpi',
      ],
      LabelFormat::DPL => [
        LabelSize::SIZE_10X15 => 'DPL_10x15_300dpi',
        LabelSize::SIZE_10X10 => 'DPL_10x10_300dpi',
      ],
    ];
    $setting = $this->settings->get();
    /** @var ?string $printingType */
    $printingType = @$mapping[$setting->getLabelFormat()][$setting->getLabelSize()];
    if (!$printingType) {
      throw new \UnexpectedValueException('Incoherent label format/size settings.');
    }
    return $printingType;
  }

}
